# Title: Contributing Area Dispersal Area Export Coefficient Model (CADA-ECM)
#        for ArcGIS
# Description: Performs a series of calculations to determine the accumulation
#              of N and P within a study site
# Software Requirements: ArcGIS 10, Spatial Analyst Extension, ArcGIS Desktop 64-bit Background Geoprocessing* (*highly recommended for larger basins; see ReadMe file for more information)
# Original Authors: Emily Stephan, Ted Endreny
# Last Edited by Original Authors: 8/30/2016
# Based on 1) Endreny, T. and E. F. Wood (2003). "Watershed Weighting of Export Coefficients to Map Critical Phosphorus Loading Areas."
#               Journal of the American Water Resources Association 39(1): 165-181.
# Based on 2) Stephan, E. A. and T. A. Endreny (2016). "Weighting Nitrogen and Phosphorus Pixel Pollutant Loads to Represent Runoff and Buffering Likelihoods."
#               JAWRA Journal of the American Water Resources Association 52(2): 336-349.
# Code further developed for automation in preparation to run CADA nationwide
# Authors of automation: James Kruegler, Robert Coville, Satoshi Hirabayashi, Jeremy Hatfield
# ===============================================================================
# Software Requirements: ArcGIS 10, Spatial Analyst Extension
# File Requirements:
#  - basinlist.csv holds the list of 8-digit USGS gage or HUC-8 ID numbers, to be read by Buffer and used sequentially as areas of interest.
#  - HUC8refs.csv holds the list of USGS gage IDs being run, along with the codes for the corresponding/overlapping HUC-8 basins. This list is used as a reference when calling CSV files of localized EC values from the /Constants/EC/LocalizedECs folder.
#  - StateSoils2.py pulls soils information and clips it to gage basin of interest. If edits need to be made as to how or what soils data is passed to CADA, this is probably the file to edit. This file is imported into CADABatch for its own calculations.
#  - /Constants/ folder with the following items:
#  	      ITEM												FOLDER				Desc.
#  	      USBasins.shp										BasinGrid			Shapefile of US Gauge Basins
#  	      gagesII_9322_sept30_2011.shp 						BasinGrid			Shapefile of Us Gauge Points
#  	      usdem 											DEM					ESRI Grid, DEM of US
#  	      NationalAverageLookup.csv							EC					National average EC values for each NLCD class
#         /LocalizedECs/ subfolder							EC					CSV files of HUC-8 specific EC values for each NLCD class in a basin
#  	      WBDHU8.shp										Huc8				Huc8 Shapefile of US
#  	      nlcd_2011_landcover_2011_edition_2014_10_10.img	LC					Landcover img
#  	      BuffTemp.mxd(and associated layers)				MapTemplate			Template of Resulting Layers
#  	      Replace.mxd										MapTemplate			Template of resulting map
#  	      cb_2017_us_state_20m.shp							States				Shapefile of US
#  	      World_UTM_Grid.lpk								UTM					UTM grid of US
#  	      ST/State.gdb for all states						StatesSoils			Soils data for each state. every state has its own folder. Requires script to join necessary tables.
#  - /basin_shapefiles/ folder, to hold custom basin shapefiles when being used
# Last Edited: 7/14/2020 by James Kruegler
# Last Edited: 2/25/2020 by T. Endreny
# ===============================================================================
# ====================
# INSTRUCTIONS FOR USE
# ====================
# Buffer script should be in the same directory as StateSoils2.py, basinlist.csv, /Constants/ folder, and (if needed) the /basin_shapefiles/ folder, as expected by the script.
# 
# 1) Open basinlist.csv and enter the 8 digit USGS gage ID # or 8-digit HUC (keep leading zeroes). The column not being used should have a 0 entered.
# --- More than one ID can be entered, each in its proper column. Columns need to be separated by a comma and rows separated by a linebreak. Make sure there are no additional commas after the row representing the last basin.
# --- Even if a custom polygon is being used that doesn't represent a USGS gage or HUC-8 basin, its shapefile needs to be associated with a nearby gage or HUC-8 ID so it is associated with the correct UTM zone.
# --- **Editing basinlist.csv in a text editor, NOT Excel, ensures the simple, expected internal file format is maintained.
# 2) Open HUC8refs.csv and, *for each USGS gage basin being run*, enter its gage ID in the first column and the 8-digit ID of an overlapping HUC-8 in the second column. Keep leading zeroes.
# --- For almost all USGS gage basins, the overlapping HUC-8 is recorded on its individual National Water Information System (NWIS) page: https://waterdata.usgs.gov/nwis/uv
# --- If not running any USGS gage basins, this file must be kept empty aside from the header row.
# --- Editing HUC8refs.csv in a text editor, NOT Excel, ensures the simple, expected internal file format is maintained.
# 3) If using localized EC values, place the CSV file of values for each basin in the /Constants/EC/LocalizedECs folder.
# --- Make sure each file has its corresponding 8-digit HUC (with or without leading zeroes) in its name.
# --- If a localized EC file does not exist for a basin being run, the national average EC values in NationalAverageLookup.csv will be used instead.
# 4) If using a custom basin shapefile(s), place them in the /basin_shapefiles/ folder.
# --- Each shapefile needs its 8-digit ID (either USGS gage or HUC) in its name.
# 5) Open Buffer python script and check 'CONFIGURATION' settings
# --- mapMode="AllMaps" prepares a .mpk map pack file showing all maps/data frames generated by the script on a single printable page.
# --- mapMode="IndvMaps" prepares individual printable maps for each map/data frame generated by the script.
# --- ArcGIS_version = "10.5" runs some ArcPy functions with the arguments needed in ArcGIS 10.5.1
# --- ArcGIS_version = "10.7" runs some ArcPy functions with the arguments needed in ArcGIS 10.7.1
# 6) Run script (SHIFT+F10).
# 
# An output folder will be created in the same directory the script is in, named according to the 8-digit ID input.
# If outputs already exist for that ID, the script may break. 
# Before re-running script for already-ran watershed, save or delete past outputs.
#
# ===========
# KNOWN ISSUE
# ===========
# The Buffer script has been known to fail when attempting to process large basins such as HUC-8 basins (Error 999998; likely from running out of local workspace memory when using Spatial Analyst tools on large rasters). 
# To avoid this, it is highly recommended users install ArcGIS Desktop 64-bit Background Geoprocessing before using Buffer's "HUC8" mode.
#
#  ===============================================================================

# ********************************************************
# CONFIGURATION: set mapMode
# ********************************************************
# "IndvMaps" for output maps as individual MXDs and JPGs; "AllMaps" for output maps combined into one MXD page. Default=IndvMaps
mapMode = "IndvMaps"  # options: "IndvMaps" or "AllMaps"
# ********************************************************

# Import system modules
import os
import shutil
import csv
import numpy
import sys
from arcpy.sa import *
from arcpy import *
from arcpy.analysis import *
from arcpy.management import *
from StateSoils2 import ssoilinfo

# Define raster grid length (m) and area (m^2); these are assumed to be the same for DEM, NLCD, Soils, etc.
RasterLength_m = 15
RasterArea_sqm = RasterLength_m*RasterLength_m

# Define ArcMap version as 10.5 or 10.7
ArcMapV = 10.5

# Get current directory to use as default path
path = os.getcwd()

# Check out the ArcGIS Spatial Analyst extension license
arcpy.CheckOutExtension("Spatial")

#Paths to: Constants folder holding input data; BasinGrid folder for output; basin_shapefiles folder for custom watersheds
cpath = path+'\\Constants\\'
bsin = cpath+'BasinGrid\\'
huc = cpath+'Huc8\\'
shppath = path+'\\basin_shapefiles\\'

#allow for arcpy to overwrite prior output
arcpy.env.overwriteOutput = True

#Set default mxd workspace and scratch gdb for feature layers (ArcMap restrictions require scratch)
mxd = mapping.MapDocument(cpath+'Template\\Bufftemp.mxd')
if mapMode == "IndvMaps": mxdf = mapping.MapDocument(cpath+'Template\\Replace.mxd')

# Create IDs vector with unique basin 8-digit ID #s (either USGS gage or HUC) that correspond to USBasins.shp file
# basinlist.csv is the input file listing basins, also called watersheds, to analyze
IDs = []
with open(path+'\\basinlist.csv') as c:
    read = csv.reader(c)
    next(read, None)
    for row in read:
        #ID is vector with values from 0th and 1st columns of reach row
        ID = [str(row[0]),str(row[1])]
        #ID vector is appended to IDs vector which can have multiple rows
        IDs.append(ID)
# Set up dictionary of USGS_gage:HUC8 entries
# For reference when running a USGS gage basin and looking for localized EC files (named by HUC8) 
USGS_to_HUC_lookup = {}
with open(path+'\\HUC8refs.csv') as c:
    reflist = list(csv.reader(c))
if len(reflist) > 1: #If the HUC8refs list isn't empty after the headers
    for row in reflist[1:]:
        # USGS gage IDs are the keys, HUC8s are the values
        USGS_to_HUC_lookup[str(row[0])] = str(row[1])

#loop through each basin ID stored in IDs vector
for row in IDs:
    #ID is non-zero ID value in row of IDs vector
    ID = [i for i in row if i != '0'][0]
    #ID_type is either USGS_gage or HUC8 corresponding to non-zero ID value
    ID_type = ['USGS_gage','HUC8'][row.index(ID)]
    try:
        #===============================================================================
        #Calculations for data input, projection and clipping starting
        #===============================================================================
        print "Database will be accessed for "+ID_type+" basin ID: "+ID
        #Establish constants for each ID, required or python may use constant defined for previous ID

        # Read in csv lookup table with Export Coefficient total phosphorus (TP) and total nitrogen (TN) values
        # Table requires field named "LC" with NLCD LC types (11, 21,.., 95) and a fields named TP_50 and TN_50 with 50th percentile ECs (kg/ha/yr)
        # If current basin ID is for a USGS gage, look up the corresponding 8-digit HUC
        if ID_type == 'USGS_gage':
            HUC8 = USGS_to_HUC_lookup[ID]
        else:
            HUC8 = ID
        # If the 8-digit HUC (stripped of leading zeroes) is found in the name of any of the /LocalizedECs/ files:
        if any(str(int(HUC8)) in file for file in os.listdir(cpath+'EC\\LocalizedECs')):
            # Identify the file with the HUC in its name
            localECfile = [file for file in os.listdir(cpath+'EC\\LocalizedECs') if str(int(HUC8)) in file][0]
            # Assign that file's absolute path to the lookup variable
            lookup = os.path.join(cpath, 'EC\\LocalizedECs', localECfile)
            print "Localized EC values pulled from "+lookup
        # If there is no localized EC file for this basin, use the national average EC values instead
        else:
            lookup = cpath+'EC\\NationalAverageLookup.csv'
            print "Using national average EC values pulled from "+lookup

        #Get landcover from 2011 NLCD, 2014 edition
        NLCD_input = Raster(cpath+'LC\\nlcd_2011_landcover_2011_edition_2014_10_10.img')
        #Get impervious cover from 2011 NLCD, 2014 edition
        IC_input = Raster(cpath +'Imperv\\nlcd_2011_impervious_2011_edition_2014_10_10.img')
        #Get DEM
        DEM_input = Raster(cpath+'DEM\usdem')
        #Get global UTM projection 
        UTMGrid = cpath+'UTM\\World_UTM_Grid.lpk'
        #Get states polygon
        States = cpath+'States\\cb_2017_us_state_20m.shp'
        #create results unique results folder and .gdb for each ID
        rpath = path+'\\Results'+ID
        if not os.path.exists(rpath):
            os.makedirs(rpath)
        #Create results.gdb and define its path and workspace
        CreateFileGDB_management (rpath, 'results.gdb')
        gdbpath = rpath+'\\results.gdb\\'
        env.workspace = rpath

        # Create basin shapefile "Basin.shp"
        # Check if alternate shapefile with USGS Gage or HUC ID has been provided in directory /basin_shapefiles
        if os.path.exists(shppath):
            try:
                shpfile = [fil for fil in os.listdir(shppath) if ID in fil and '.shp' in fil and '.xml' not in fil][0]
                CopyFeatures_management(shppath+shpfile,rpath+'\\Basin.shp')
                print "Basin shapefile copied from custom file "+shppath+shpfile
            except IndexError:
                shpfile = False
        if shpfile == False: #Define query for Select_analysis
            if ID_type == 'USGS_gage':
                query = """ "GAGE_ID" = '%s' """%ID
                Select_analysis(bsin+'USBasinsShape\USBasins.shp','Basin',query) #Create Basin.shp from query match with polygon within USBasins.shp
                print "Basin shapefile extracted from USBasins.shp in "+bsin
            if ID_type == 'HUC8':
                query = """ "HUC8" = '%s' """%ID
                Select_analysis(huc+'WBDHU8.shp','Basin',query) #Create Basin.shp from query match with polygon within WBDHU8.shp
                print "Basin shapefile extracted from WBDHU8.shp in "+huc
        # Make copies of the starting basin shapefile, for all GCS projections used by input rasters
        arcpy.Project_management('Basin.shp', 'BasinNLCD', arcpy.Describe(NLCD_input).spatialReference)
        arcpy.Project_management('Basin.shp', 'BasinIC', arcpy.Describe(IC_input).spatialReference)
        arcpy.Project_management('Basin.shp', 'BasinDEM', arcpy.Describe(DEM_input).spatialReference)
        print "Created polygons BasinNLCD, BasinIC, and BasinDEM"

        for basin in ['BasinNLCD.shp', 'BasinIC.shp', 'BasinDEM.shp']:
            # Describe spatial extent of each basin in their own distance units
            ext = arcpy.Describe(basin).extent
            # Establish thickness of border beyond original extents; 10% of original total height and width
            border_width = ext.width*0.1
            border_height = ext.height*0.1
            # Place new extent dimensions in a string together
            newext = "%s %s %s %s" % (ext.XMin-border_width, ext.YMin-border_height, ext.XMax+border_width, ext.YMax+border_height)
            names = {'BasinNLCD.shp':[NLCD_input, 'NLCD'], 'BasinIC.shp':[IC_input, 'IC'], 'BasinDEM.shp':[DEM_input, 'DEM']}
            arcpy.Clip_management(names[basin][0], newext, 'box'+names[basin][1], '', '', '', "NO_MAINTAIN_EXTENT")
            print "Created raster box"+names[basin][1]

        # Split Intersect_analysis into two parts, as GIS licensing limits 2-inputs at a time
        Intersect_analysis([UTMGrid,States],'UTMstates','ALL')
        Intersect_analysis(['Basin.shp','UTMstates.shp'],'Metadata','ALL') 
        MakeFeatureLayer_management('Metadata.shp',gdbpath+'Metalyr', "", '', "")
        #Extract watershed's intersecting UTM zone and state(s) from the available metadata
        metadata = SearchCursor(gdbpath+'Metalyr','','','Zone;STUSPS')
        States = []
        for vals in metadata:
            Zone= int(vals.getValue('Zone'))
            States.append(str(vals.getValue('STUSPS')))
        #Define spatial reference for zone in UTM
        sr = int(str(269) + str(Zone))
        arcpy.Project_management('Basin.shp', 'BasinPCS.shp', arcpy.SpatialReference(sr))
        print "Created polygon BasinPCS.shp"

        #Project box clips of LC (integer code), IC (%), and DEM (m) to UTM feature; all rasters to standard grid size (RasterLength_m)
        arcpy.ProjectRaster_management('boxNLCD', 'boxNLCD_PCS', arcpy.SpatialReference(sr), "NEAREST", RasterLength_m)
        print "Created raster boxNLCD_PCS"
        # Ensuring all three rasters have their cells aligned by giving them identical bottom-left corners
        ext = arcpy.Describe('boxNLCD_PCS').extent
        anchor_point = "%s %s" % (ext.lowerLeft.X, ext.lowerLeft.Y)
        arcpy.ProjectRaster_management('boxIC', 'boxIC_PCS', arcpy.SpatialReference(sr), "NEAREST", RasterLength_m, '', anchor_point)
        print "Created raster boxIC_PCS"
        arcpy.ProjectRaster_management('boxDEM', 'boxDEM_PCS', arcpy.SpatialReference(sr), "BILINEAR", RasterLength_m, '', anchor_point)
        print "Created raster boxDEM_PCS"

        #Clip the UTM-projected box clips of LC, IC, & DEM to BasinPCS.shp
        arcpy.Clip_management('boxNLCD_PCS', '#', 'NLCD', 'BasinPCS.shp', '', 'ClippingGeometry')
        print "Created raster LC"
        arcpy.Clip_management('boxIC_PCS', '#', 'IC', 'BasinPCS.shp', '', 'ClippingGeometry')
        print "Created raster IC"
        arcpy.Clip_management('boxDEM_PCS', '#', 'DEM', 'BasinPCS.shp', '', 'ClippingGeometry')
        print "Created raster DEM"

        #Call StatesSoils2.py: Use pre-processed SSURGO data for state(s) in "States" and extract soils data for basin of interest
        ssoilinfo(States,'Basin.shp',ID)
        print "Created polygon SoilArea in StatesSoils2.py"
        #Take the extracted SSURGO data in 'SoilArea' and project to the correct UTM zone
        arcpy.Project_management(gdbpath+'SoilArea', gdbpath+'SoilPCS', arcpy.SpatialReference(sr), '')
        print "Created raster SoilPCS"

        env.workspace = rpath

        print "SSURGO soil layers will be extracted and converted to rasters"
        #Extract from SSURGO data several layers of data containing soil physical properties; then check that soil data grids are aligned with other input rasters
        #Establish rectangular extent of the NLCD raster, to use in the re-alignment of SSURGO rasters
        ext = arcpy.Describe('NLCD').extent
        box = "%s %s %s %s" % (ext.XMin, ext.YMin, ext.XMax, ext.YMax)
        #Convert soil depth (cm) from feature to raster, to standard grid size
        #SSURGO variable hzdepb_r represents distance from the top of the soil to the base of the soil horizon (or, the start of the restrictive layer constraining flow)
        depthtif = arcpy.FeatureToRaster_conversion(gdbpath+'SoilPCS','hzdepb_r', 'Depth', RasterLength_m)
        arcpy.Clip_management('Depth', box, 'S_Depth', 'NLCD', "", "NONE", "MAINTAIN_EXTENT")
        print "Created raster S_Depth"
        #Convert soil hydraulic conductivity (micrometer/s) from feature to raster, to standard grid size (RasterLength_m)
        ksattif = arcpy.FeatureToRaster_conversion(gdbpath+'SoilPCS','ksat_r', 'Ksat', RasterLength_m)
        arcpy.Clip_management('Ksat', box, 'S_Ksat', 'NLCD', "", "NONE", "MAINTAIN_EXTENT")
        print "Created raster S_Ksat"
        #Values needed for f_param calculation: high and low estimates of Ksat (micrometer/s) and soil thickness (cm)
        #Convert upper layer soil hydraulic conductivity (micrometer/s) from feature to raster, to standard grid size        
        ksat_high = arcpy.FeatureToRaster_conversion(gdbpath+'SoilPCS','ksat_h', 'KsatH', RasterLength_m)
        arcpy.Clip_management('KsatH', box, 'S_KsatH', 'NLCD', "", "NONE", "MAINTAIN_EXTENT")
        print "Created raster S_KsatH"
        #Convert lower layer soil hydraulic conductivity (micrometer/s) from feature to raster, to standard grid size        
        ksat_low = arcpy.FeatureToRaster_conversion(gdbpath+'SoilPCS','ksat_l', 'KsatL', RasterLength_m)
        arcpy.Clip_management('KsatL', box, 'S_KsatL', 'NLCD', "", "NONE", "MAINTAIN_EXTENT")
        print "Created raster S_KsatL"
        #Convert soil thickness (cm) from feature to raster, to standard grid size        
        soil_thickness = arcpy.FeatureToRaster_conversion(gdbpath+'SoilPCS','hzthk_r', 'Thick', RasterLength_m)
        arcpy.Clip_management('Thick', box, 'S_Thick', 'NLCD', "", "NONE", "MAINTAIN_EXTENT")
        print "Created raster S_Thick"
        #Convert minimum depth to water table (cm) from feature to raster, to standard grid size
        wtdepthtif = arcpy.FeatureToRaster_conversion(gdbpath+'SoilPCS','wtdepannmin', 'WTable', RasterLength_m)
        arcpy.Clip_management('WTable', box, 'S_WTable', 'NLCD', "", "NONE", "MAINTAIN_EXTENT")
        print "Created raster S_WTable"

        #===============================================================================
        #Calculations for data input, projection and clipping ending
        #===============================================================================
        
        #===============================================================================
        #Calculations for SSURGO terms starting
        #===============================================================================
        print "Filling gaps in SSURGO database with ArcMapV "+str(ArcMapV) 
        # Depth to the bottom of the soil horizon (cm)
        #Create temporary raster ddd to examine for gaps
        ddd = Raster("S_Depth")
        print "Created raster ddd as temporary SSURGO rasters"
 		# IsNull returns 1 if NoData, returns 0 otherwise. Used with Con, LHS is RHS_2nd_term if RHS_1st_term is NoData, LHS is RHS_3rd_term otherwise
        gap = Con(IsNull(ddd),1,0)
        print "Created raster gap to identify SSURGO NoData grids"

        if ArcMapV == 10.5:
            # For ArcGIS 10.5.1
            arcpy.Clip_management(gap, "", "S_Depth_gap", "BasinPCS.shp", "255", "ClippingGeometry")
            arcpy.RasterToPolygon_conversion("S_Depth_gap", "S_Depth_gap.shp", "NO_SIMPLIFY", "VALUE")
            arcpy.Dissolve_management("S_Depth_gap.shp", "S_Depth_gap_pcnt.shp", "GRIDCODE", "", "MULTI_PART", "DISSOLVE_LINES")
            arcpy.AddGeometryAttributes_management("S_Depth_gap_pcnt.shp", "AREA", "", "", "")
            #EucAllocation and .save commands created AllocDep by replacing NoData cells in raster S_Depth with nearest cell data value
            alloc = EucAllocation("S_Depth", "", "", RasterLength_m, "VALUE", "", "")
            alloc.save("AllocDep")

        if ArcMapV == 10.7:
            # For ArcGIS 10.7.1
            arcpy.Clip_management(gap, "", "S_Depth_gap", "BasinPCS.shp", "255", "ClippingGeometry")
            arcpy.RasterToPolygon_conversion("S_Depth_gap", "S_Depth_gap_pcnt.shp", "NO_SIMPLIFY", "VALUE", "MULTIPLE_OUTER_PART" )
            arcpy.AddGeometryAttributes_management("S_Depth_gap_pcnt.shp", "AREA", "", "", "")
            # Allocate the value for each cell in AllocDep,from the nearest S_Depth value based on Euclidean distance.
            arcpy.BuildRasterAttributeTable_management("S_Depth", "Overwrite")
            #EucAllocation and .save commands created AllocDep by replacing NoData cells in raster S_Depth with nearest cell data value
            alloc = EucAllocation("S_Depth", "", "", RasterLength_m, "VALUE", "", "")
            alloc.save("AllocDep")
            print "Created raster AllocDep to fill SSURGO NoData grids using Euclidian interpolation"

		# IsNull returns 1 if NoData, returns 0 otherwise. Used with Con, LHS is RHS_2nd_term if RHS_1st_term is NoData, LHS is RHS_3rd_term otherwise
        est = Con(IsNull(ddd),"AllocDep",ddd)
        print "Created raster est that used AllocDep to replace NoData in temporary SSURGO raster ddd"
        print "Creation of ddd, gap, AllocDep and est repeated for each SSURGO raster, as listed next:"

        #S_Depth_est created to fill any gaps in S_Depth SSURGO data
        arcpy.Clip_management(est, "", "S_Depth_est", "BasinPCS.shp", "255", "ClippingGeometry")
        print "Created raster S_Depth_est"
        
        # KSat (representative) (micrometer/s)
        # Identify gap percentage
        ddd = Raster("S_Ksat")
		# IsNull returns 1 if NoData, returns 0 otherwise. Used with Con, LHS is RHS_2nd_term if RHS_1st_term is NoData, LHS is RHS_3rd_term otherwise
        gap = Con(IsNull(ddd),1,0)
 
        if ArcMapV == 10.5:
            # For ArcGIS 10.5.1
            arcpy.Clip_management(gap, "", "S_Ksat_gap", "BasinPCS.shp", "255", "ClippingGeometry")
            arcpy.RasterToPolygon_conversion("S_Ksat_gap", "S_Ksat_gap.shp", "NO_SIMPLIFY", "VALUE")
            arcpy.Dissolve_management("S_Ksat_gap.shp", "S_Ksat_gap_pcnt.shp", "GRIDCODE", "", "MULTI_PART", "DISSOLVE_LINES")
            arcpy.AddGeometryAttributes_management("S_Ksat_gap_pcnt.shp", "AREA", "", "", "")
            # Convert from micrometer float to integer with extra accuracy
            mult=1e5
            S_Ksat_int = Int(ddd * mult)
			#Evaluate S_Ksat_gap, if they are 1, set to NoData, otherwise set to S_Ksat_int
            S_Ks_gapfix = SetNull("S_Ksat_gap", S_Ksat_int, "VALUE = 1")
            S_Ks_gapfix.save("S_Ksat_int")
            # Allocate the value for each cell in AllocKsat,from the nearest S_Ksat value based on Euclidean distance.
            alloc = EucAllocation("S_Ksat_int", "", "", RasterLength_m, "VALUE", "", "")
            alloc.save("AllocKsat")

        if ArcMapV == 10.7:
            # For ArcGIS 10.7.1
            arcpy.Clip_management(gap, "", "S_Ksat_gap", "BasinPCS.shp", "255", "ClippingGeometry")
            arcpy.RasterToPolygon_conversion("S_Ksat_gap", "S_Ksat_gap_pcnt.shp", "NO_SIMPLIFY", "VALUE", "MULTIPLE_OUTER_PART")
            arcpy.AddGeometryAttributes_management("S_Ksat_gap_pcnt.shp", "AREA", "", "", "")
            # Convert from micrometer float to integer with extra accuracy
            mult=1e5
            S_Ksat_int = Int(ddd * mult)
			#Evaluate S_Ksat_gap, if they are 1, set to NoData, otherwise set to S_Ksat_int
            S_Ks_gapfix = SetNull("S_Ksat_gap", S_Ksat_int, "VALUE = 1")
            S_Ks_gapfix.save("S_Ksat_int")
            # Allocate the value for each cell in AllocKsat,from the nearest S_Ksat value based on Euclidean distance.
            arcpy.BuildRasterAttributeTable_management("S_Ksat_int", "Overwrite")
            alloc = EucAllocation("S_Ksat_int", "", "", RasterLength_m, "VALUE", "", "")
            alloc.save("AllocKsat")

		# IsNull returns 1 if NoData, returns 0 otherwise. Used with Con, LHS is RHS_2nd_term if RHS_1st_term is NoData, LHS is RHS_3rd_term otherwise
        est = Con(IsNull("S_Ksat_int"),"AllocKsat","S_Ksat_int")
		#convert from integer to float
        est = Float(est)/mult

        #S_Ksat_est created to fill any gaps in SSURGO data
        arcpy.Clip_management(est, "", "S_Ksat_est", "BasinPCS.shp", "255", "ClippingGeometry")
        print "Created raster S_Ksat_est"
        S_Ksat_stat = arcpy.CalculateStatistics_management('S_Ksat_est')
        S_Ksat_avg = float(str(arcpy.GetRasterProperties_management(S_Ksat_stat, "MEAN")))
        #convert from micrometer/s to cm/hr
        print "Mean SSURGO soil hydraulic conductivity (cm/hr): "+str(S_Ksat_avg*1e-4*3600)

        # KSat (high) (micrometer/s)
        # Identify gap percentage
        ddd = Raster("S_KsatH")
		# IsNull returns 1 if NoData, returns 0 otherwise. Used with Con, LHS is RHS_2nd_term if RHS_1st_term is NoData, LHS is RHS_3rd_term otherwise
        gap = Con(IsNull(ddd),1,0)

        if ArcMapV == 10.5:
            # For ArcGIS 10.5.1
            arcpy.Clip_management(gap, "", "S_KsatH_gap", "BasinPCS.shp", "255", "ClippingGeometry")
            arcpy.RasterToPolygon_conversion("S_KsatH_gap", "S_KsatH_gap.shp", "NO_SIMPLIFY", "VALUE")
            arcpy.Dissolve_management("S_KsatH_gap.shp", "S_KsatH_gap_pcnt.shp", "GRIDCODE", "", "MULTI_PART", "DISSOLVE_LINES")
            arcpy.AddGeometryAttributes_management("S_KsatH_gap_pcnt.shp", "AREA", "", "", "")
            # use multiplier=100,000 to covert from the floating point to a decimal number
            S_KsatH_int = Int(ddd * mult)
			#Evaluate S_KsatH_gap, if they are 1, set to NoData, otherwise set to S_KsatH_int
            S_Ks_gapfix = SetNull("S_KsatH_gap", S_KsatH_int, "VALUE = 1")
            S_Ks_gapfix.save("S_KsatH_int")
            # Allocate the value for each cell in AllocKsat_H,from the nearest S_KsatH_int value based on Euclidean distance.
            alloc = EucAllocation("S_KsatH_int", "", "", RasterLength_m, "VALUE", "", "")
            alloc.save("AllocKsat_H")

        if ArcMapV == 10.7:
            # For ArcGIS 10.7.1
            arcpy.Clip_management(gap, "", "S_KsatH_gap", "BasinPCS.shp", "255", "ClippingGeometry")
            arcpy.RasterToPolygon_conversion("S_KsatH_gap", "S_KsatH_gap_pcnt.shp", "NO_SIMPLIFY", "VALUE", "MULTIPLE_OUTER_PART")
            arcpy.AddGeometryAttributes_management("S_KsatH_gap_pcnt.shp", "AREA", "", "", "")
            # Convert from floating point to integer raster
            S_KsatH_int = Int(ddd * mult)
			#Evaluate S_KsatH_gap, if they are 1, set to NoData, otherwise set to S_KsatH_int
            S_Ks_gapfix = SetNull("S_KsatH_gap", S_KsatH_int, "VALUE = 1")
            S_Ks_gapfix.save("S_KsatH_int")
            # Allocate the value for each cell in AllocKsat,from the nearest S_Ksat value based on Euclidean distance.
            arcpy.BuildRasterAttributeTable_management("S_KsatH_int", "Overwrite")
            print "Created attribute table for S_KsatH_int"
            alloc = EucAllocation("S_KsatH_int", "", "", RasterLength_m, "VALUE", "", "")
            alloc.save("AllocKsat_H")

		# IsNull returns 1 if NoData, returns 0 otherwise. Used with Con, LHS is RHS_2nd_term if RHS_1st_term is NoData, LHS is RHS_3rd_term otherwise
        est = Con(IsNull("S_KsatH_int"),"AllocKsat_H","S_KsatH_int")
		#convert from integer to float
        est = Float(est)/mult

        #S_KsatH_est created to fill any gaps in SSURGO data
        arcpy.Clip_management(est, "", "S_KsatH_est", "BasinPCS.shp", "255", "ClippingGeometry")
        print "Created raster S_KsatH_est"

        # KSat (low) (micrometer/s)
        # Identify gap percentage
        ddd = Raster("S_KsatL")
		# IsNull returns 1 if NoData, returns 0 otherwise. Used with Con, LHS is RHS_2nd_term if RHS_1st_term is NoData, LHS is RHS_3rd_term otherwise
        gap = Con(IsNull(ddd),1,0)

        if ArcMapV == 10.5:
            # For ArcGIS 10.5.1
            arcpy.Clip_management(gap, "", "S_KsatL_gap", "BasinPCS.shp", "255", "ClippingGeometry")
            arcpy.RasterToPolygon_conversion("S_KsatL_gap", "S_KsatL_gap.shp", "NO_SIMPLIFY", "VALUE")
            arcpy.Dissolve_management("S_KsatL_gap.shp", "S_KsatL_gap_pcnt.shp", "GRIDCODE", "", "MULTI_PART", "DISSOLVE_LINES")
            arcpy.AddGeometryAttributes_management("S_KsatL_gap_pcnt.shp", "AREA", "", "", "")
            # use multiplier=100,000 to covert from the floating point to a decimal number
            S_KsatL_int = Int(ddd * mult)
			#Evaluate S_KsatL_gap, if they are 1, set to NoData, otherwise set to S_KsatL_int
            S_Ks_gapfix = SetNull("S_KsatL_gap", S_KsatL_int, "VALUE = 1")
            S_Ks_gapfix.save("S_KsatL_int")
            # Allocate the value for each cell in AllocKsat_L,from the nearest S_KsatL_int value based on Euclidean distance.
            alloc = EucAllocation("S_KsatL_int", "", "", RasterLength_m, "VALUE", "", "")
            alloc.save("AllocKsat_L")

        if ArcMapV == 10.7:
            # For ArcGIS 10.7.1
            arcpy.Clip_management(gap, "", "S_KsatL_gap", "BasinPCS.shp", "255", "ClippingGeometry")
            arcpy.RasterToPolygon_conversion("S_KsatL_gap", "S_KsatL_gap_pcnt.shp", "NO_SIMPLIFY", "VALUE", "MULTIPLE_OUTER_PART")
            arcpy.AddGeometryAttributes_management("S_KsatL_gap_pcnt.shp", "AREA", "", "", "")
            # Convert from floating point to integer raster
            S_KsatL_int = Int(ddd * mult)
			#Evaluate S_KsatL_gap, if they are 1, set to NoData, otherwise set to S_KsatL_int
            S_Ks_gapfix = SetNull("S_KsatL_gap", S_KsatL_int, "VALUE = 1")
            S_Ks_gapfix.save("S_KsatL_int")
            # Allocate the value for each cell in AllocKsat,from the nearest S_Ksat value based on Euclidean distance.
            arcpy.BuildRasterAttributeTable_management("S_KsatL_int", "Overwrite")
            print "Created attribute table for S_KsatL_int"
            alloc = EucAllocation("S_KsatL_int", "", "", RasterLength_m, "VALUE", "", "")
            alloc.save("AllocKsat_L")

		# IsNull returns 1 if NoData, returns 0 otherwise. Used with Con, LHS is RHS_2nd_term if RHS_1st_term is NoData, LHS is RHS_3rd_term otherwise
        est = Con(IsNull("S_KsatL_int"),"AllocKsat_L","S_KsatL_int")
		#convert from integer to float
        est = Float(est)/mult

        #S_KsatL_est created to fill any gaps in SSURGO data
        arcpy.Clip_management(est, "", "S_KsatL_est", "BasinPCS.shp", "255", "ClippingGeometry")
        print "Created raster S_KsatL_est"

        # Soil thickness (cm)
        # Identify gap percentage
        ddd = Raster("S_Thick")
		# IsNull returns 1 if NoData, returns 0 otherwise. Used with Con, LHS is RHS_2nd_term if RHS_1st_term is NoData, LHS is RHS_3rd_term otherwise
        gap = Con(IsNull(ddd),1,0)

        if ArcMapV == 10.5:
            # For ArcGIS 10.5.1
            arcpy.Clip_management(gap, "", "S_Thick_gap", "BasinPCS.shp", "255", "ClippingGeometry")
            arcpy.RasterToPolygon_conversion("S_Thick_gap", "S_Thick_gap.shp", "NO_SIMPLIFY", "VALUE")
            arcpy.Dissolve_management("S_Thick_gap.shp", "S_Thick_gap_pcnt.shp", "GRIDCODE", "", "MULTI_PART", "DISSOLVE_LINES")
            arcpy.AddGeometryAttributes_management("S_Thick_gap_pcnt.shp", "AREA", "", "", "")
            # Allocate the value for each cell in AllocThk,from the nearest S_Thick value based on Euclidean distance.
            alloc = EucAllocation("S_Thick", "", "", RasterLength_m, "VALUE", "", "")
            alloc.save("AllocThk")

        if ArcMapV == 10.7:
            # For ArcGIS 10.7.1
            arcpy.Clip_management(gap, "", "S_Thick_gap", "BasinPCS.shp", "255", "ClippingGeometry")
            arcpy.RasterToPolygon_conversion("S_Thick_gap", "S_Thick_gap_pcnt.shp", "NO_SIMPLIFY", "VALUE", "MULTIPLE_OUTER_PART" )
            arcpy.AddGeometryAttributes_management("S_Thick_gap_pcnt.shp", "AREA", "", "", "")
            # Allocate the value for each cell in AllocDep,from the nearest S_Thick value based on Euclidean distance.
            arcpy.BuildRasterAttributeTable_management("S_Thick", "Overwrite")
            print "Created attribute table for S_Thick"
            alloc = EucAllocation("S_Thick", "", "", RasterLength_m, "VALUE", "", "")
            alloc.save("AllocThk")

		# IsNull returns 1 if NoData, returns 0 otherwise. Used with Con, LHS is RHS_2nd_term if RHS_1st_term is NoData, LHS is RHS_3rd_term otherwise
        est = Con(IsNull(ddd),"AllocThk",ddd)

        #S_Thick_est created to fill any gaps in SSURGO data
        arcpy.Clip_management(est, "", "S_Thick_est", "BasinPCS.shp", "255", "ClippingGeometry")
        print "Created raster S_Thick_est"
        S_Thick_stat = arcpy.CalculateStatistics_management('S_Thick_est')
        S_Thick_avg = float(str(arcpy.GetRasterProperties_management(S_Thick_stat, "MEAN")))
        print "Mean SSURGO soil depth (cm): "+str(S_Thick_avg)

        # Water table minimum depth (cm)
        # Identify gap percentage
        ddd = Raster("S_WTable")
		# IsNull returns 1 if NoData, returns 0 otherwise. Used with Con, LHS is RHS_2nd_term if RHS_1st_term is NoData, LHS is RHS_3rd_term otherwise
        gap = Con(IsNull(ddd),1,0)

        if ArcMapV == 10.5:
            # For ArcGIS 10.5.1
            arcpy.Clip_management(gap, "", "S_WTable_gap", "BasinPCS.shp", "255", "ClippingGeometry")
            arcpy.RasterToPolygon_conversion("S_WTable_gap", "S_WTable_gap.shp", "NO_SIMPLIFY", "VALUE")
            arcpy.Dissolve_management("S_WTable_gap.shp", "S_WTable_gap_pcnt.shp", "GRIDCODE", "", "MULTI_PART", "DISSOLVE_LINES")
            arcpy.AddGeometryAttributes_management("S_WTable_gap_pcnt.shp", "AREA", "", "", "")
            # Allocate the value for each cell in AllocWT,from the nearest S_WTable value based on Euclidean distance.
            alloc = EucAllocation("S_WTable", "", "", RasterLength_m, "VALUE", "", "")
            alloc.save("AllocWT")

        if ArcMapV == 10.7:
            # For ArcGIS 10.7.1
            arcpy.Clip_management(gap, "", "S_WTable_gap", "BasinPCS.shp", "255", "ClippingGeometry")
            arcpy.RasterToPolygon_conversion("S_WTable_gap", "S_WTable_gap_pcnt.shp", "NO_SIMPLIFY", "VALUE", "MULTIPLE_OUTER_PART" )
            arcpy.AddGeometryAttributes_management("S_WTable_gap_pcnt.shp", "AREA", "", "", "")
           # Allocate the value for each cell in AllocWT,from the nearest S_WTable value based on Euclidean distance.
            arcpy.BuildRasterAttributeTable_management("S_WTable", "Overwrite")
            print "Created attribute table for S_WTable"
            alloc = EucAllocation("S_WTable", "", "", RasterLength_m, "VALUE", "", "")
            alloc.save("AllocWT")

		# IsNull returns 1 if NoData, returns 0 otherwise. Used with Con, LHS is RHS_2nd_term if RHS_1st_term is NoData, LHS is RHS_3rd_term otherwise
        est = Con(IsNull(ddd),"AllocWT",ddd)

        #S_WTable_est created to fill any gaps in SSURGO data
        arcpy.Clip_management(est, "", "S_WTable_est", "BasinPCS.shp", "255", "ClippingGeometry")
        print "Created raster S_WTable_est"
        S_WTable_stat = arcpy.CalculateStatistics_management('S_WTable_est')
        S_WTable_avg = float(str(arcpy.GetRasterProperties_management(S_WTable_stat, "MEAN")))
        print "Mean groundwater table depth extracted from SSURGO data (cm): "+str(S_WTable_avg)
       
        #Decay of soil transmissivity with depth (1/cm)
        #Defined using Equations 6&7 of Williams et al., The Water Availability Tool ... Scientific Investigations Report 2009-5248, https://pubs.usgs.gov/sir/2009/5248/
        print "Calculating transmissivity decay f_param"
        conmult = Divide("S_KsatH_est", "S_KsatL_est")
        conmult.save("conmult")
        print "Created conmult raster"
        f_rast = Divide(Ln(conmult), "S_Thick_est")
        print "Created f_rast raster"
        f_int = arcpy.Clip_management(Int(f_rast*1e9), "", "f_int", "BasinPCS.shp", "", "ClippingGeometry")
        print "Created f_int raster"
        f_table = ZonalStatisticsAsTable("BasinPCS.shp", "FID", "f_int", "f_table", "DATA", "MEDIAN")
        print "Created f_table table"
        f_cursor = arcpy.SearchCursor("f_table", "", "", "MEDIAN")
        print "Created f_cursor from reading table"
        for vals in f_cursor:
            #dividing 1e9 to undue prior multiplication of f_rast by 1e9
            f_param = (vals.getValue("MEDIAN"))/1e9
        print "SSURGO-derived TOPMODEL scaling parameter f_param value:"+str(f_param)

        #===============================================================================
        #Calculations for SSURGO terms ending
        #===============================================================================

        #===============================================================================
        #Outputs section starting
        #===============================================================================
        #Components of DEM map layer
        demdf = mapping.ListDataFrames(mxd)[1]
        demlyr = mapping.Layer('DEM')
        templatedem = mapping.ListLayers(mxd)[1]
        mapping.UpdateLayer(demdf, demlyr, templatedem, symbology_only = True)
        #For IndvMaps mode, build DEM map and export as PDF
        if mapMode == "IndvMaps":
            mapping.AddLayer(mapping.ListDataFrames(mxdf)[0],demlyr)
            # title = mapping.ListLayoutElements(mxdf, "TEXT_ELEMENT")[0]
            # title.text='Basin'+str(ID)+'DEM'
            for legend in mapping.ListLayoutElements(mxdf,"LEGEND_ELEMENT"): #Add map legend, describing only the DEM
                legend.title = ""
            mxdf.saveACopy(rpath+'\\' +str(ID)+'DEM'+'.mxd') #Save DEM dataframe in its own MXD file
            copy = mapping.MapDocument(rpath+'\\' +str(ID)+'DEM'+'.mxd')
            mapping.ExportToJPEG (copy,rpath+'\\' +str(ID)+'DEM'+'.pdf' )
            for lyr in mapping.ListLayers(mxdf):
                mapping.RemoveLayer(mapping.ListDataFrames(mxdf)[0],lyr)
        #For AllMaps mode, add DEM layer to MXD workspace
        elif mapMode == "AllMaps":
            mapping.AddLayer(demdf,demlyr)
            mapping.RemoveLayer(demdf, templatedem)

        #Components of land cover map layer
        lcdf = mapping.ListDataFrames(mxd)[0]
        lclyr = mapping.Layer('NLCD')
        templatelc = mapping.ListLayers(mxd)[0]
        mapping.UpdateLayer(lcdf,lclyr,templatelc, symbology_only = True)
        #For IndvMaps mode, build land cover map and export as PDF
        if mapMode == "IndvMaps":
            mapping.AddLayer(mapping.ListDataFrames(mxdf)[0],lclyr)
            # title = mapping.ListLayoutElements(mxdf, "TEXT_ELEMENT")[0]
            # title.text='Basin'+str(ID)+'Landcover'
            for legend in mapping.ListLayoutElements(mxdf,"LEGEND_ELEMENT"): #Add map legend, describing only the land cover
                legend.title = ""
                legend.adjustColumnCount(4)
            mxdf.saveACopy(rpath+'\\' +str(ID)+'LC'+'.mxd') #Save land cover dataframe in its own MXD file
            copy = mapping.MapDocument(rpath+'\\' +str(ID)+'LC'+'.mxd')
            mapping.ExportToJPEG (copy,rpath+'\\' +str(ID)+'LC'+'.pdf' )
            for lyr in mapping.ListLayers(mxdf):
                mapping.RemoveLayer(mapping.ListDataFrames(mxdf)[0],lyr)
        #For AllMaps mode, add land cover layer to MXD workspace
        elif mapMode == "AllMaps":
            mapping.AddLayer(lcdf,lclyr)
            mapping.RemoveLayer(lcdf, templatelc)

        #===============================================================================
        #Outputs section ending
        #===============================================================================

        #===============================================================================
        #Calculations for runoff index likelihood starting
        #===============================================================================
        #Get DEM_fill with Spatial Analyst function to fill sinks that do not flow to neighboring grid
        arcpy.gp.Fill_sa('DEM','DEM_fill', "")
        print "Created raster DEM_fill"
        #Get DEM_fdir DEM with Spatial Analyst function to assign flow directions; default is D8, option MF, and D-Infinity
        arcpy.gp.FlowDirection_sa('DEM_fill','DEM_fdir', "FORCE",'nill')
        print "Created raster DEM_fdir"
        #Get DEM_facc DEM with Spatial Analyst function to count all other grids flowing into each grid; no weighting grid used
        arcpy.gp.FlowAccumulation_sa('DEM_fdir','DEM_facc', "", "FLOAT")
        print "Created raster DEM_facc"
        #Threshold value for designating river cells; Arc Hydro GIS suggests 1% of watershed-max DEM_facc value as a starting point
        RiverThresh_FA = float(arcpy.GetRasterProperties_management('DEM_facc', "MAXIMUM").getOutput(0))*0.01
        print "Created raster RiverThresh_FA"
        print "DEM layers made"

        #Create River grid when flow accumulation grid exceeds RiverThresh_FA
        #Conditional (raster, value if true, value if false, condition tested)
        #river0 grid will have 0 in river cells, and RasterLength_m in non-river cells
        #river01 grid will have 0 in river cells, and 1 in non-river cells
        #Used in both map modes
        river0 = Con('DEM_facc', 0, RasterLength_m, "VALUE > " + str(RiverThresh_FA))
        river0.save("river0")
        print "Created river0 raster"
        river01 = Con('DEM_facc', 0, 1, "VALUE > " + str(RiverThresh_FA))
        river01.save("river01")
        print "Created river01 raster"

        #Compute DEM slope as percent
        DEM_slp_int = Int(Slope("DEM_fill","PERCENT_RISE")*1e5)
        DEM_slp_int.save("DEM_slp_int")
        print "Created raster DEM_slp_int"
        #Checking that the DEM_slp_int raster value attribute table (VAT) is built
        BuildRasterAttributeTable_management("DEM_slp_int", "NONE")
        print "Created attribute table for DEM_slp_int"

        #Find minimum DEM slope to replace slopes smaller than threshold of 0.01%. 
        DEM_slp_min = 0.01
        print "Created variable DEM_slp_min"
        print 'Min slope (%): '+str(DEM_slp_min)
		#Replace slopes values below a threshold slope of 0.01%*1e5 = 1000 with DEM_slp_min*1e5
        DEM_slp_int = Con(DEM_slp_int, DEM_slp_min*1e5, DEM_slp_int, "VALUE < 1000")
        print "Updated raster DEM_slp_int"
		#Convert slope raster to float fraction slope from integer percent slope by multiplying with 1e-7. 
        #DEM_slope = tan(B) = rise/run, where B is local pixel slope in degrees
        DEM_slope = Clip_management(Times(DEM_slp_int, 1e-7),'#','DEM_slope','BasinPCS.shp','','ClippingGeometry')
        print "Created raster DEM_slope"

        #Computing a weighting grid of impervious and pervious areas per contour width for use in flow accumulation
        #Using Eq 6 of Stephan and Endreny (2016)
        imp_percent = Lookup('IC', "VALUE")
        print "Created raster imp_percent"
        #imp_weight describes total impervious area per pixel width (m)
        imp_weight = Times(imp_percent,0.01)*RasterArea_sqm/RasterLength_m
        imp_weight.save("imp_weight")
        print "Created raster imp_weight"
        #Pervious cover (%) as complement to impervious
        per_percent = 100-imp_percent
        print "Created raster per_percent"
        #per_weight describes total pervious area per pixel width (m)
        per_weight = Times(per_percent,0.01)*RasterArea_sqm/RasterLength_m
        per_weight.save("per_weight")
        print "Created raster per_weight"

        #Compute RI_surf_i in Eq 6 of Stephan and Endreny (2016), from impervious Topographic Index, Eq 8 of Valeo, C. et al. (2000). Journal of Hydrology 228(1-2): 68-81.
		#Eq 6 is RI_surf_i = ln(FlowAccumulation (impervious surfaces) / DEM_slope) = ln(FA_imp / DEM_slope), converted to integer
        #Step #1 of 1
        #Convert to Integer by multiplying 1e4
        #FlowAccumulation function uses imp_weight as weighting grid of contributing area, and then adds imp_weight for local cell
        FA_imp = (FlowAccumulation(FlowDirection('DEM_fill','FORCE'),'imp_weight'))+'imp_weight'
        FA_imp.save('FA_imp')
        RI_surf_int = Int(Ln(Divide('FA_imp','DEM_slope'))*1e4)
        RI_surf_int.save('RI_surf_int')
        print "Created raster RI_surf_int"
        BuildRasterAttributeTable_management('RI_surf_int', "NONE")
        print "Created attribute table for RI_surf_int"
        #Convert RI_surf_int NoData values to 0. NoData occurs due to ln(0), which happens when there is no impervious area in the cell and FA_imp = 0
		# IsNull returns 1 if NoData, returns 0 otherwise. Used with Con, LHS is RHS_2nd_term if RHS_1st_term is NoData, LHS is RHS_3rd_term otherwise
        RI_surf_int0 = Clip_management(Con(IsNull('RI_surf_int'),0,'RI_surf_int'),'#','RI_surf_int0','BasinPCS.shp','','ClippingGeometry')
        print "Created raster RI_surf_int0"
 
        #Rescale the RI_surf_int distribution to have a minimum of 1, which will remove negatives and normalize indexes for cross-comparison 
        if int(arcpy.GetRasterProperties_management("RI_surf_int0", "MINIMUM", "").getOutput(0)) != 10000:
            RI_surf_min = int(arcpy.GetRasterProperties_management("RI_surf_int0", "MINIMUM", "").getOutput(0))
            RI_surf_difference = RI_surf_min - 10000
            RI_surf_nrm = Minus("RI_surf_int0", RI_surf_difference)
            RI_surf_nrm.save("RI_surf_nrm")
            print "Rescaled raster RI_surf_nrm has minimum value of 1e4"
            #Convert RI_surf_nrm to Float by multiplying 1e-4, clip to Basin shape
            RI_surf_i = Clip_management(Times(RI_surf_nrm, 1e-4),'#','RI_surf_i','BasinPCS.shp','','ClippingGeometry')
        else:
            #Convert RI_surf_int0 to Float by multiplying 1e-4, clip to Basin shape
            RI_surf_i = Clip_management(Times(RI_surf_int0, 1e-4),'#','RI_surf_i','BasinPCS.shp','','ClippingGeometry')
        print "Created raster RI_surf_i"

        #Compute RI_sub in Eq 7 of Stephan and Endreny (2016), from Soil Topographic Index, Eq 10 in Sivapalan, M., et al. (1987). Water Resources Research 23(12): 2266-2278.
        #   and Eq 1 in Endreny, T. A., et al. (2000). Hydrological Processes 14(1): 1-20.
        #Eq 7 uses average and local transmissivity, S_Trans_avg and S_Trans, and flow accumulation, FA_per, and DEM_slope, where FA_per = a and DEM_slope = tan(B)
        #Eq 7 separated into 2 Eqs, RI_sub = ln[(S_Trans_avg * FA_per) / (S_Trans * DEM_slope)] = ln(FA_per / DEM_slope) + ln(S_Trans_avg / S_Trans)  
        #Step #1 of 3
        #Compute Eq 7 1st RHS term ln(FA_per / DEM_slope), where FA_per = a and DEM_slope = tan(B)
        #FlowAccumulation function uses per_weight as weighting grid of contributing area, and then adds per_weight for local cell
        FA_per = (FlowAccumulation(FlowDirection('DEM_fill','FORCE'),'per_weight'))+'per_weight'
        FA_per.save('FA_per')
        print "Created raster FA_per"
		#Convert to integer by multiplying by 1e4, making conditional statements easier
        RI_sub_RHS1 = Int(Ln(Divide('FA_per','DEM_slope'))*1e4)
        RI_sub_RHS1.save("RI_sub_RHS1")
        print "Created raster ln(a/tanB) 1st term in STI, as RI_sub_RHS1"
        #Step #2 of 3
        #Compute Eq 7 2nd RHS term ln(S_Trans_avg / S_Trans) using soil depth (cm) and hydraulic conductivity (micrometers/s)
        #Convert transmissivity to m^2/day from (cm*um)/s using factor 0.000864 = (1/10^6)*((60*60*24)/1)*(1/100) = m/um * s/day * m/cm
        S_Trans = Times(0.000864,(Times('S_Depth_est','S_Ksat_est')))
        S_Trans.save("S_Trans")
        print "Created raster S_Trans"
        #Compute mean transmissivity
        S_Trans_avg = float(str(arcpy.GetRasterProperties_management(S_Trans, "MEAN")))
        print "Created raster S_Trans_avg"
		#Convert to integer by multiplying by 1e4, making conditional statements easier
        RI_sub_RHS2 = Int(Ln(Divide(S_Trans_avg,S_Trans))*1e4)
        RI_sub_RHS2.save('RI_sub_RHS2')
        print "Created raster ln(T_avg/Ti) 2nd term in STI, as RI_sub_RHS2"
        #Step #3 of 3
        #Compute Eq 7 by combining 2 RHS terms, which remains in integer 1e4
        RI_sub_int = RI_sub_RHS1 + RI_sub_RHS2
        RI_sub_int.save('RI_sub_int')
        print "Created raster RI_sub_int"
        BuildRasterAttributeTable_management('RI_sub_int', "NONE")
        print "Created attribute table for RI_sub_int"
        #Convert RI_sub_int NoData values to 0. NoData occurs due to ln(0), which happens when there is no pervious area in the cell and FA_per = 0
		# IsNull returns 1 if NoData, returns 0 otherwise. Used with Con, LHS is RHS_2nd_term if RHS_1st_term is NoData, LHS is RHS_3rd_term otherwise
        RI_sub_int0 = Clip_management(Con(IsNull('RI_sub_int'),0,'RI_sub_int'),'#','RI_sub_int0','BasinPCS.shp','','ClippingGeometry')
        print "Created raster RI_sub_int0"

        #Rescale the RI_sub_int0 distribution to have a minimum of 1, which will remove negatives and normalize indexes for cross-comparison 
        if int(arcpy.GetRasterProperties_management("RI_sub_int0", "MINIMUM", "").getOutput(0)) != 10000:
            RI_sub_min = int(arcpy.GetRasterProperties_management("RI_sub_int0", "MINIMUM", "").getOutput(0))
            RI_sub_difference = RI_sub_min - 10000
            RI_sub_nrm = Minus("RI_sub_int0", RI_sub_difference)
            RI_sub_nrm.save("RI_sub_nrm")
            print "Rescaled raster RI_sub_nrm has minimum value of 1e4"
            #Convert RI_sub_int0 to Float by multiplying 1e-4, clip to Basin shape
            RI_sub_i = Clip_management(Times(RI_sub_nrm, 1e-4),'#','RI_sub_i','BasinPCS.shp','','ClippingGeometry')
        else:
            #Convert RI_sub_int0 to Float by multiplying 1e-4, clip to Basin shape
            RI_sub_i = Clip_management(Times(RI_sub_int0, 1e-4),'#','RI_sub_i','BasinPCS.shp','','ClippingGeometry')
        print "Created raster RI_sub_i"

        #Mean value of soil topographic index (STI) raster for pervious flow
        arcpy.CalculateStatistics_management('RI_sub_i')
        RI_sub_avg = float(str(arcpy.GetRasterProperties_management('RI_sub_i', "MEAN")))
        print "Created raster RI_sub_avg"
        print "Mean pervious Soil Topographic Index as RI_sub_i: "+str(RI_sub_avg)
        #Mean value of  topographic index (TI) raster for impervious flow
        arcpy.CalculateStatistics_management('RI_surf_i')
        RI_surf_avg = float(str(arcpy.GetRasterProperties_management('RI_surf_i', "MEAN")))
        print "Created raster RI_surf_avg"
        print "Mean impervious Topographic Index as RI_surf_i: "+str(RI_surf_avg)
        #===============================================================================
        #Calculations for runoff index likelihood ending
        #===============================================================================

        #===============================================================================
        #Outputs section starting
        #===============================================================================
        #Components of RI_sub_i map layer
        tidf = mapping.ListDataFrames(mxd)[4]
        tilyr = mapping.Layer('RI_sub_i')
        templateti =mapping.ListLayers(mxd)[4]
        mapping.UpdateLayer(tidf,tilyr,templateti, symbology_only = True)
        #For IndvMaps mode, build map of TI_per and export as PDF
        if mapMode == "IndvMaps":
            mapping.AddLayer(mapping.ListDataFrames(mxdf)[0],tilyr)
            # title = mapping.ListLayoutElements(mxdf, "TEXT_ELEMENT")[0]
            # title.text='Basin'+str(ID)+'TI'
            for legend in mapping.ListLayoutElements(mxdf,"LEGEND_ELEMENT"): #Add map legend, describing only the TI_per layer
                legend.title = ID_type+": "+ID
            mxdf.saveACopy(rpath+'\\' +str(ID)+'TI'+'.mxd') #Save TI_per dataframe in its own MXD file
            copy = mapping.MapDocument(rpath+'\\' +str(ID)+'TI'+'.mxd')
            mapping.ExportToJPEG (copy,rpath+'\\' +str(ID)+'TI'+'.pdf' )
            for lyr in mapping.ListLayers(mxdf):
                mapping.RemoveLayer(mapping.ListDataFrames(mxdf)[0],lyr)
        #For AllMaps mode, add TI_per layer to MXD workspace
        elif mapMode == "AllMaps":
            mapping.AddLayer(tidf,tilyr)
            mapping.RemoveLayer(tidf, templateti)
        #===============================================================================
        #Outputs section ending
        #===============================================================================


        #===============================================================================
        #Calculations for buffer index slopes
        #===============================================================================
        #Compute negated filled DEM, as defined in Eq 10 in Stephan and Endreny (2016) 
        DEM_neg = Times(('DEM_fill'),-1)
        DEM_neg.save("DEM_neg")
        print "Created raster DEM_neg"

        #Get DEM_fdir DEM_neg with Spatial Analyst function to assign flow directions; default is D8, option MF, and D-Infinity
        arcpy.gp.FlowDirection_sa('DEM_neg','DEM_neg_fdir', "FORCE",'nill')
        print "Created raster DEM_neg_fdir"
        #Get DEM_facc DEM with Spatial Analyst function to count all other grids flowing into each grid; no weighting grid used
        arcpy.gp.FlowAccumulation_sa('DEM_neg_fdir','DEM_neg_facc', "", "FLOAT")
        print "Created raster DEM_neg_facc"
        #Threshold value for designating river cells; Arc Hydro GIS suggests 1% of watershed-max DEM_facc value as a starting point
        RiverThresh_FA_neg = float(arcpy.GetRasterProperties_management('DEM_neg_facc', "MAXIMUM").getOutput(0))*0.01
        print "Created raster RiverThresh_FA_neg"
        print "DEM_neg layers made"
		
        ridges0 = Con('DEM_neg_facc', 0, RasterLength_m, "VALUE > " + str(RiverThresh_FA_neg))
        ridges0.save("ridges0")

        #Define dispersal area slope as ratio of rise over run between local grid and river grid
        #Compute Buffer_rise
        #Step #1 of 3
        #Compute Buffer_rise for each local grid 
        Buffer_rise1 = Times((Slope(DEM_neg, "PERCENT_RISE"))/100,RasterLength_m)
        print "Created raster Buffer_rise1"
        #Step #2 of 3
        #Set Buffer_rise to 0 in river cells 
        Buffer_rise2 = Con(river0, 0, Buffer_rise1, "VALUE = 0")
        print "Created raster Buffer_rise2"
        #Step #3 of 3
        #Accumulate rise between local grid and river grid, and add local grid Buffer_rise
        Buffer_rise = (FlowAccumulation(FlowDirection(DEM_neg), Buffer_rise2))+Buffer_rise2
        print "Created raster Buffer_rise"

        #Compute Buffer_run
        #Step #1 of 2
        #Compute distance with FlowAccumulation and weighting grid of river0 which has value of RasterLength_m for non-river grids
        Buffer_run = (FlowAccumulation(FlowDirection(DEM_neg),river0))+river0
        print "Created raster Buffer_run"
        #Step #2 of 2
        #Compute Buffer_run to include value of 1 for all river grids, and otherwise remove 0 values to ensure no division by zero
        #Raster value is Buffer_run, condition is if it is equal to 0 (river), true then Buffer_run = 1, false then Buffer_run = Buffer_run
        Buffer_run = Con(Buffer_run, 1, Buffer_run, "VALUE = 0")
        Buffer_run.save("Buffer_run")
        print "Recreated raster Buffer_run"
        
        #Compute Buffer_slp as final slope used in computations, with river = to DEM_slp_min/100
        #Raster value is river0, condition is if it is equal to 0 (river), true then Buffer_slp = DEM_slp_min/100, false then Buffer_slp = Buffer_rise/Buffer_run
        Buffer_slp = Con(river0, DEM_slp_min/100,Divide(Buffer_rise,Buffer_run),"VALUE = 0")
        Buffer_slp.save("Buffer_slp")
        print "Created raster Buffer_slp"
        #===============================================================================
        #Calculations for buffer index pausing
        #===============================================================================

        #================================================================================
        #Calculations for buffer index flow times
        #================================================================================
        #Join lookup table of Nitrogen (N) and Phosphorous (P) export coeficient values (kg/m^2/yr) with NLCD data
        #Convert NLCD grid to polygon, then create a feature layer for that polygon
        arcpy.RasterToPolygon_conversion('NLCD', 'LC.shp', "NO_SIMPLIFY", 'VALUE')
        print "Created polygon LC.shp"
        
        arcpy.MakeFeatureLayer_management('LC.shp','LClyr', "", '', "")
        print "Created layer LClyr"
        
        #Merge lookup table values with new NLCD layer, then add the updated LClyr to the geodatabase
        arcpy.AddJoin_management('LClyr', "gridcode", lookup, "LC", "KEEP_ALL")
        arcpy.FeatureClassToFeatureClass_conversion('LClyr', gdbpath, 'LClyr', '', '', '')

        #Create Manning n roughness value raster grid of size RasterLength_m
        #n (unitless) values from Table 8.1 in Wurbs and James, Water Resources Engineering, ISBN-13: 978-0130812933, 2001, Prentice Hall, NJ
        Manning_n = arcpy.FeatureToRaster_conversion(gdbpath+'LClyr', 'n', 'Manning_n_r', RasterLength_m)
        Manning_n = arcpy.FeatureToRaster_conversion(gdbpath+'LClyr', 'n', 'Manning_n_r', RasterLength_m)
        print "Created raster Manning_n"
		
        #Create Manning R hydraulic radius value (ft) raster grid of size RasterLength_m
        #R (ft) values from Table 8.1 in Wurbs and James, Water Resources Engineering, ISBN-13: 978-0130812933, 2001, Prentice Hall, NJ
        Hydraulic_R = arcpy.FeatureToRaster_conversion(gdbpath+'LClyr', 'R', 'Hydraulic_R_r', RasterLength_m)
        print "Created raster Hydraulic_R"

        #Compute surface velocity (ft/s) of runoff for local grid using Manning Eq 9 in Stephan and Endreny (2016) (see header for full citation)
        #Unit conversion 1.49 is for BG units, Hydraulic_R_r = Hydraulic Radius (ft), DEM_slope = Slope (fraction)
        Vel_surf = Times(Divide(1.49,Raster('Manning_n_r')),Times(Raster('Hydraulic_R_r')**2/3,Raster('DEM_slope')**.5))
        Vel_surf.save("Vel_surf")
        print "Created raster Vel_surf"

        #Compute Time_surf (s) surface time for runoff to flow across the local grid (m), converted to ft (3.281 ft/m)
        Time_surf = Int(Divide(RasterLength_m * 3.281,Vel_surf)) #Time_surf_i is in seconds
        Time_surf.save("Time_surf")
        print "Created raster Time_surf"

        #Surface BI computed based on Stephan and Endreny (2016) Eq 10
        #Recompute Time_surf_r to set river cells to 0 travel time, as they don't contribute to filtering
        Time_surf_r = Int(Con(river0, 0, Time_surf, "VALUE = 0"))
        Time_surf_r.save("Time_surf_r")
        print "Created raster Time_surf_r"
        
		#Compute Time_surf_t as total accumulated travel time across the dispersal area, add in local travel time to pixel
        Time_surf_t = (FlowAccumulation(FlowDirection(DEM_neg),Time_surf_r))+Time_surf_r
        Time_surf_t.save("Time_surf_t")
        print "Created raster Time_surf_t"

		#Adjust Time_surf_t to value of 0 for all river cells, do not change other cells
        Time_surf_i = Time_surf_t * river01
        Time_surf_i.save("Time_surf_i")
        print "Created raster Time_surf_i"

        #Calculate local Buffer index as natural logarithm of time 
		#Eq 10 in Stephan and Endreny (2016), but take natural log to reduce magnitude range
        BI_surf_int = Int(Ln(Time_surf_i)*1e4)
        BI_surf_int.save("BI_surf_int")
        print "Created raster BI_surf_int"
        BuildRasterAttributeTable_management('BI_surf_int', "NONE")
        print "Created attribute table for BI_surf_int"
        #Rescale the BI_surf_int distribution to have a minimum of 1, which will remove negatives and normalize indexes for cross-comparison 
        if int(arcpy.GetRasterProperties_management("BI_surf_int", "MINIMUM", "").getOutput(0)) != 10000:
            BI_surf_min = int(arcpy.GetRasterProperties_management("BI_surf_int", "MINIMUM", "").getOutput(0))
            BI_surf_difference = BI_surf_min - 10000
            BI_surf_nrm = Minus("BI_surf_int", BI_surf_difference)
            # Removing river cells and basin fringe cells with nonsensical negative values
            BI_surf_nrm = SetNull(BI_surf_nrm, BI_surf_nrm, "VALUE < 10000")
            BI_surf_nrm.save("BI_surf_nrm")
            print "Rescaled raster BI_surf_nrm has minimum value of 1e4"
            #Convert BI_surf_nrm to Float by multiplying 1e-4, clip to Basin shape
            BI_surf_i = Clip_management(Times(BI_surf_nrm, 1e-4),'#','BI_surf_i','BasinPCS.shp','','ClippingGeometry')
        else:
            #Convert BI_surf_int to Float by multiplying 1e-4, clip to Basin shape
            BI_surf_i = Clip_management(Times(BI_surf_int, 1e-4),'#','BI_surf_i','BasinPCS.shp','','ClippingGeometry')
        print "Created raster BI_surf_i"

        #Compute S_WTable (m) as groundwater table depth using Eq 13 of Stephan and Endreny (2016)
        #SSURGO data provided S_WTable_avg (cm) and f_param, DEM data provided RI_sub_i and RI_sub_avg, aka topographic index
        #Dividing by 100 to convert from cm to m
        S_WTable = Divide((S_WTable_avg - Times((1/f_param),(Raster('RI_sub_i') - RI_sub_avg))),100)
        S_WTable.save("S_WTable")
        print "Created raster S_WTable using TI theory"
        T_WTable_stat = arcpy.CalculateStatistics_management('S_WTable')
        T_WTable_avg = float(str(arcpy.GetRasterProperties_management(T_WTable_stat, "MEAN")))
        #convert from m to cm
        print "Mean groundwater table depth derived from TI theory (using SSURGO and DEM data)(cm): "+str(T_WTable_avg*100)

        #Compute WTable_slope (fraction) using Slope percent_rise function and dividing by 100
        WTable_slope = Divide(Slope(S_WTable,"PERCENT_RISE"),100)
        # WTable_slope = Con(WTable_slope,1e-4,WTable_slope,"VALUE<1e-4")
        WTable_slope.save("WTable_slope")
        print "Created raster WTable_slope" 
        
        #Compute Vel_sub (m/s) subsurface velocity for runoff using Darcy Eq 12 of Stephan and Endreny (2016) (see header for full citation)
        #Unit conversion for hydraulic conductivity (um/s) multiplied by 1E-6 m/um
        Vel_sub = Times(1e-6,Times('S_Ksat_est','WTable_slope'))
        #Setting Vel_sub to a minimum 1e-5m/s or 0.8m/day to avoid creating null data values in Time_sub
        Vel_sub = Con(Vel_sub,1e-5,Vel_sub,"VALUE<1e-5")
        Vel_sub = Con(IsNull(Vel_sub),1e-5,Vel_sub)
        Clip_management(Vel_sub,'#','Vel_sub','BasinPCS.shp','','ClippingGeometry')
        print "Created raster Vel_sub"
        
        #Compute Time_sub (s) subsurface time for runoff as RasterLength_m / subsurface velocity
        #Null values are created in this division if very small Vel_sub dividing into WTable_slopes
        Time_sub = Int(Divide(RasterLength_m, Vel_sub))
        Time_sub.save("Time_sub")
        print "Created raster Time_sub"
        
        #Subsurface BI computed based on Stephan and Endreny (2016)
        #Recompute Time_sub_r to set river cells to 0 travel time, as they don't contribute to filtering
        Time_sub_r = Int(Con(river0, 0, Time_sub, "VALUE = 0"))
        Time_sub_r.save("Time_sub_r")
        print "Created raster Time_sub_r"

		#Compute Time_sub_t as total accumulated travel time across the dispersal area, add in local travel time to pixel
        Time_sub_t = (FlowAccumulation(FlowDirection(DEM_neg),Time_sub_r))+Time_sub_r
        Time_sub_t.save("Time_sub_t")
        print "Created raster Time_sub_t"

		#Adjust Time_sub_t to value of 0 for all river cells, do not change other cells
        Time_sub_i = Time_sub_t * river01
        Time_sub_i.save("Time_sub_i")
        print "Recreated raster Time_sub_i"

        #Calculate local Buffer index as natural logarithm of time 
		#Eq 10 in Stephan and Endreny (2016), but take natural log to reduce magnitude range
        BI_sub_int = Int(Ln(Time_sub_i)*1e4)
        BI_sub_int.save("BI_sub_int")
        print "Created raster BI_sub_int"
        BuildRasterAttributeTable_management('BI_sub_int', "NONE")
        print "Created attribute table for BI_sub_int"
        #Rescale the BI_sub_int distribution to have a minimum of 1, which will remove negatives and normalize indexes for cross-comparison 
        if int(arcpy.GetRasterProperties_management("BI_sub_int", "MINIMUM", "").getOutput(0)) != 10000:
            BI_sub_min = int(arcpy.GetRasterProperties_management("BI_sub_int", "MINIMUM", "").getOutput(0))
            BI_sub_difference = BI_sub_min - 10000
            BI_sub_nrm = Minus("BI_sub_int", BI_sub_difference)
            # Removing river cells and basin fringe cells with nonsensical negative values
            BI_sub_nrm = SetNull(BI_sub_nrm, BI_sub_nrm, "VALUE < 10000")
            BI_sub_nrm.save("BI_sub_nrm")
            print "Rescaled raster BI_sub_nrm has minimum value of 1e4"
            #Convert BI_sub_nrm to Float by multiplying 1e-4, clip to Basin shape
            BI_sub_i = Clip_management(Times(BI_sub_nrm, 1e-4),'#','BI_sub_i','BasinPCS.shp','','ClippingGeometry')
        else:
            #Convert BI_sub_int to Float by multiplying 1e-4, clip to Basin shape
            BI_sub_i = Clip_management(Times(BI_sub_int, 1e-4),'#','BI_sub_i','BasinPCS.shp','','ClippingGeometry')
        print "Created raster BI_sub_i"

        #================================================================================
        #Calculations for buffer index flow times ending
        #================================================================================

        #================================================================================
        #Calculations for buffer index ratios
        #================================================================================
        #Calculate BI_surf_i statistics
        arcpy.CalculateStatistics_management(BI_surf_i)
        #Extract the mean from the statistics and convert to a floating point number
        BI_surf_avg = float(str(arcpy.GetRasterProperties_management(BI_surf_i, "MEAN")))
        print "Created BI_surf_avg"
        print "Mean surface Buffer Index: "+str(BI_surf_avg)

        #Calculate BI_sub_i statistics
        arcpy.CalculateStatistics_management(BI_sub_i)
        #Extract the mean from the statistics and convert to a floating point number
        BI_sub_avg = float(str(arcpy.GetRasterProperties_management(BI_sub_i, "MEAN")))
        print "Created BI_sub_avg"
        print "Mean subsurface Buffer Index: "+str(BI_sub_avg)

		#Buffering Index (BI) is ratio of BI_average to BI_local, large ratios lead to larger weights for Export Coefficient
		#Compute ratios of BI_surf_ratio and BI_sub_ratio as 3rd RHS terms in Eq 1 and 2 from Stephan and Endreny (2016)
        BI_surf_ratio = Divide(BI_surf_avg,BI_surf_i)
        BI_surf_ratio.save("BI_surf_ratio")
        print "Created raster BI_surf_ratio"
        BI_sub_ratio = Divide(BI_sub_avg,BI_sub_i)
        BI_sub_ratio.save("BI_sub_ratio")
        print "Created raster BI_sub_ratio"
        #================================================================================
        #Calculations for buffer index ratios ending
        #================================================================================

        #===============================================================================
        #Outputs section starting
        #===============================================================================
        ###Generate image of normalized BI for display
        #Components of BIsurf map layer
        bisurfdf = mapping.ListDataFrames(mxd)[5]
        bisurflyr = mapping.Layer('BI_surf_ratio')
        templatebisurf =mapping.ListLayers(mxd)[5]
        mapping.UpdateLayer(bisurfdf,bisurflyr,templatebisurf, symbology_only = True)
        #For IndvMaps mode, build map of BIsurf and export as PDF
        if mapMode == "IndvMaps":
            mapping.AddLayer(mapping.ListDataFrames(mxdf)[0],bisurflyr)
            # title = mapping.ListLayoutElements(mxdf, "TEXT_ELEMENT")[0]
            # title.text='Basin'+str(ID)+'SurfaceBI'
            for legend in mapping.ListLayoutElements(mxdf,"LEGEND_ELEMENT"): #Add map legend, describing only the BIsurf layer
                legend.title = ID_type+": "+ID
            mxdf.saveACopy(rpath+'\\' +str(ID)+'BIsurf'+'.mxd') #Save BIsurf dataframe in its own MXD file
            copy = mapping.MapDocument(rpath+'\\' +str(ID)+'BIsurf'+'.mxd')
            mapping.ExportToJPEG (copy,rpath+'\\' +str(ID)+'BIsurf'+'.pdf' )
            for lyr in mapping.ListLayers(mxdf):
                mapping.RemoveLayer(mapping.ListDataFrames(mxdf)[0],lyr)
        #For AllMaps mode, add BIsurf layer to MXD workspace
        elif mapMode == "AllMaps":
            mapping.AddLayer(bisurfdf,bisurflyr)
            mapping.RemoveLayer(bisurfdf,templatebisurf)

        #Components of BIsub map layer
        bisubdf = mapping.ListDataFrames(mxd)[6]
        bisublyr = mapping.Layer('BI_sub_ratio')
        templatebisub =mapping.ListLayers(mxd)[6]
        mapping.UpdateLayer(bisubdf,bisublyr,templatebisub, symbology_only = True)
        #For IndvMaps mode, build map of BIsub and export as PDF
        if mapMode == "IndvMaps":
            mapping.AddLayer(mapping.ListDataFrames(mxdf)[0],bisublyr)
            # title = mapping.ListLayoutElements(mxdf, "TEXT_ELEMENT")[0]
            # title.text='Basin'+str(ID)+'SubSurfaceBI'
            for legend in mapping.ListLayoutElements(mxdf,"LEGEND_ELEMENT"): #Add map legend, describing only the BIsub layer
                legend.title = ID_type+": "+ID
            mxdf.saveACopy(rpath+'\\' +str(ID)+'BIsub'+'.mxd') #Save BIsub dataframe in its own MXD file
            copy = mapping.MapDocument(rpath+'\\' +str(ID)+'BIsub'+'.mxd')
            mapping.ExportToJPEG (copy,rpath+'\\' +str(ID)+'BIsub'+'.pdf' )
            for lyr in mapping.ListLayers(mxdf):
                mapping.RemoveLayer(mapping.ListDataFrames(mxdf)[0],lyr)
        #For AllMaps mode, add BIsub layer to MXD workspace
        elif mapMode == "AllMaps":
            mapping.AddLayer(bisubdf,bisublyr)
            mapping.RemoveLayer(bisubdf, templatebisub)
        #===============================================================================
        #Outputs section ending
        #===============================================================================

        #================================================================================
        #Calculations for runoff index ratios
        #================================================================================
		#Runoff Index (RI)
		#Compute ratios of RI_surf_i and RI_sub_i as 2nd RHS terms in Eq 1 and 2 from Stephan and Endreny (2016)
        RI_sub_ratio = Divide('RI_sub_i',RI_sub_avg)
        RI_sub_ratio.save("RI_sub_ratio")
        print "Created raster RI_sub_ratio"
        RI_surf_ratio = Divide('RI_surf_i',RI_surf_avg)
        RI_surf_ratio.save("RI_surf_ratio")
        print "Created raster RI_surf_ratio"

        #================================================================================
        #Calculations for Export Coefficient values unweighted
        #================================================================================
		#Export Coefficient values for total nitrogen (TN) and total phosphorus (TP), unweighted, from White et al 
		#Used in Eq 1 with RI_surf_ratio and BI_surf_ratio, in Eq 2 with RI_sub_ratio and BI_sub_ratio, from Stephan and Endreny (2016)
        EC_TN = arcpy.FeatureToRaster_conversion(gdbpath+'LClyr', 'TN_50', 'lyrTN', RasterLength_m)
        print "Created raster EC_TN"
        EC_TP = arcpy.FeatureToRaster_conversion(gdbpath+'LClyr', 'TP_50', 'lyrTP', RasterLength_m)
        print "Created raster EC_TP"
        #================================================================================
        #Calculations for Export Coefficient values unweighted ending
        #================================================================================

        #===============================================================================
        #Outputs section starting
        #===============================================================================
        #Components of TN map layer
        tndf = mapping.ListDataFrames(mxd)[3]
        tnlyr = mapping.Layer('lyrTN')
        templatetn =mapping.ListLayers(mxd)[3]
        mapping.UpdateLayer(tndf,tnlyr,templatetn, symbology_only = True)
        #For IndvMaps mode, build map of TN and export as PDF
        if mapMode == "IndvMaps":
            mapping.AddLayer(mapping.ListDataFrames(mxdf)[0],tnlyr)
            # title = mapping.ListLayoutElements(mxdf, "TEXT_ELEMENT")[0]
            # title.text='Basin'+str(ID)+'TotalNitrogen'
            for legend in mapping.ListLayoutElements(mxdf,"LEGEND_ELEMENT"): #Add map legend, describing only the TN layer
                legend.title = ID_type+": "+ID
            mxdf.saveACopy(rpath+'\\' +str(ID)+'TN'+'.mxd') #Save TN dataframe in its own MXD file
            copy = mapping.MapDocument(rpath+'\\' +str(ID)+'TN'+'.mxd')
            mapping.ExportToJPEG (copy,rpath+'\\' +str(ID)+'TN'+'.pdf' )
            for lyr in mapping.ListLayers(mxdf):
                mapping.RemoveLayer(mapping.ListDataFrames(mxdf)[0],lyr)
        #For AllMaps mode, add TN layer to MXD workspace
        elif mapMode == "AllMaps":
            mapping.AddLayer(tndf,tnlyr)
            mapping.RemoveLayer(tndf, templatetn)

        #Components of TP map layer
        tpdf = mapping.ListDataFrames(mxd)[2]
        tplyr = mapping.Layer('lyrTP')
        templatetp =mapping.ListLayers(mxd)[2]
        mapping.UpdateLayer(tpdf,tplyr,templatetp, symbology_only = True)
        #For IndvMaps mode, build map of TP and export as PDF
        if mapMode == "IndvMaps":
            mapping.AddLayer(mapping.ListDataFrames(mxdf)[0],tplyr)
            # title = mapping.ListLayoutElements(mxdf, "TEXT_ELEMENT")[0]
            # title.text='Basin'+str(ID)+'TotalPhosphorous'
            for legend in mapping.ListLayoutElements(mxdf,"LEGEND_ELEMENT"): #Add map legend, describing only the TP layer
                legend.title = ID_type+": "+ID
            mxdf.saveACopy(rpath+'\\' +str(ID)+'TP'+'.mxd') #Save TP dataframe in its own MXD file
            copy = mapping.MapDocument(rpath+'\\' +str(ID)+'TP'+'.mxd')
            mapping.ExportToJPEG (copy,rpath+'\\' +str(ID)+'TP'+'.pdf' )
            for lyr in mapping.ListLayers(mxdf):
                mapping.RemoveLayer(mapping.ListDataFrames(mxdf)[0],lyr)
        #For AllMaps mode, add TP layer to MXD workspace
        elif mapMode == "AllMaps":
            mapping.AddLayer(tpdf,tplyr)
            mapping.RemoveLayer(tpdf, templatetp)
        #===============================================================================
        #Outputs section ending
        #===============================================================================

        #================================================================================
        #Calculations for Export Coefficient values weighted
        #================================================================================
		#Weighted Export Coefficient values for total nitrogen (TN) and total phosphorus (TP)
		#Computing Eq 1 with RI_surf_ratio and BI_surf_ratio from Stephan and Endreny (2016)       
        EC_TP_wtd = Times(Times(BI_surf_ratio,RI_surf_ratio),EC_TP)
        EC_TP_wtd.save("EC_TP_wtd")
        print "Created raster EC_TP_wtd"

		#Computing Eq 2 with RI_sub_ratio, BI_sub_ratio from Stephan and Endreny (2016)       
        EC_TN_wtd = Times(Times(BI_sub_ratio,RI_sub_ratio),EC_TN)
        EC_TN_wtd.save("EC_TN_wtd")
        print "Created raster EC_TN_wtd"
        #================================================================================
        #Calculations for Export Coefficient values weighted ending
        #================================================================================

        #===============================================================================
        #Outputs section starting
        #===============================================================================
        #Components of ECdTN map layer
        ECDTNdf = mapping.ListDataFrames(mxd)[8]
        ECDTNlyr = mapping.Layer('EC_TN_wtd')
        templateECDTN =mapping.ListLayers(mxd)[8]
        mapping.UpdateLayer(ECDTNdf,ECDTNlyr,templateECDTN, symbology_only = True)
        #For IndvMaps mode, build map of ECdTN and export as PDF
        if mapMode == "IndvMaps":
            mapping.AddLayer(mapping.ListDataFrames(mxdf)[0],ECDTNlyr)
            # title = mapping.ListLayoutElements(mxdf, "TEXT_ELEMENT")[0]
            # title.text='Basin'+str(ID)+'EC Change in Nitrogen'
            for legend in mapping.ListLayoutElements(mxdf,"LEGEND_ELEMENT"): #Add map legend, describing only the ECdTN layer
                legend.title = ""
            mxdf.saveACopy(rpath+'\\' +str(ID)+'ECdTN'+'.mxd') #Save ECdTN dataframe in its own MXD file
            copy = mapping.MapDocument(rpath+'\\' +str(ID)+'ECdTN'+'.mxd')
            mapping.ExportToJPEG (copy,rpath+'\\' +str(ID)+'ECdTN'+'.pdf' )
            for lyr in mapping.ListLayers(mxdf):
                mapping.RemoveLayer(mapping.ListDataFrames(mxdf)[0],lyr)
        #For AllMaps mode, add ECdTN layer to MXD workspace
        elif mapMode == "AllMaps":
            mapping.AddLayer(ECDTNdf,ECDTNlyr)
            mapping.RemoveLayer(ECDTNdf, templateECDTN)

        #Components of ECdTP map layer
        ECDTPdf = mapping.ListDataFrames(mxd)[7]
        ECDTPlyr = mapping.Layer('EC_TP_wtd')
        templateECDTP =mapping.ListLayers(mxd)[7]
        mapping.UpdateLayer(ECDTPdf,ECDTPlyr,templateECDTP, symbology_only = True)
        #For IndvMaps mode, build map of ECdTP and export as PDF
        if mapMode == "IndvMaps":
            mapping.AddLayer(mapping.ListDataFrames(mxdf)[0],ECDTPlyr)
            # title = mapping.ListLayoutElements(mxdf, "TEXT_ELEMENT")[0]
            # title.text='Basin'+str(ID)+'EC Change in Phosphorous'
            for legend in mapping.ListLayoutElements(mxdf,"LEGEND_ELEMENT"): #Add map legend, describing only the ECdTP layer
                legend.title = ""
            mxdf.saveACopy(rpath+'\\' +str(ID)+'ECdTP'+'.mxd') #Save ECdTP dataframe in its own MXD file
            copy = mapping.MapDocument(rpath+'\\' +str(ID)+'ECdTP'+'.mxd')
            mapping.ExportToJPEG (copy,rpath+'\\' +str(ID)+'ECdTP'+'.pdf' )
            for lyr in mapping.ListLayers(mxdf):
                mapping.RemoveLayer(mapping.ListDataFrames(mxdf)[0],lyr)
        #For AllMaps mode, add ECdTP layer to MXD workspace
        elif mapMode == "AllMaps":
            mapping.AddLayer(ECDTPdf,ECDTPlyr)
            mapping.RemoveLayer(ECDTPdf, templateECDTP)

            mxd.saveACopy(rpath+'\\Buffer'+str(ID)+'.mxd')
        #===============================================================================
        #Outputs section ending
        #===============================================================================

        #Clean Up Files and Variables:
        arcpy.Delete_management('Basin.shp')
        arcpy.Delete_management('BasinNLCD')
        arcpy.Delete_management('BasinIC')
        arcpy.Delete_management('BasinDEM')
        arcpy.Delete_management('boxNLCD')
        arcpy.Delete_management('boxIC')
        arcpy.Delete_management('boxDEM')
        arcpy.Delete_management('boxNLCD_PCS')
        arcpy.Delete_management('boxIC_PCS')
        arcpy.Delete_management('boxDEM_PCS')
        arcpy.Delete_management('Ksat')
        arcpy.Delete_management('Depth')
        arcpy.Delete_management('KsatH')
        arcpy.Delete_management('KsatL')
        arcpy.Delete_management('Thick')
        arcpy.Delete_management('WTable')
        arcpy.Delete_management('DEM_fdir')
        arcpy.Delete_management('DEM_fill')
        arcpy.Delete_management('meters')
        arcpy.Delete_management('ddd')
        arcpy.Delete_management('gap')
        arcpy.Delete_management('est')
        arcpy.Delete_management('alloc')
        arcpy.Delete_management('S_Depth_gap')
        arcpy.Delete_management('S_Ksat')
        arcpy.Delete_management('S_KsatH_gap')
        arcpy.Delete_management('S_KsatL_gap')
        arcpy.Delete_management('S_Thick_gap')
        arcpy.Delete_management('S_WTable_gap')
        arcpy.Delete_management('S_Depth_gap.shp')
        arcpy.Delete_management('S_Ksat.shp')
        arcpy.Delete_management('S_KsatH_gap.shp')
        arcpy.Delete_management('S_KsatL_gap.shp')
        arcpy.Delete_management('S_Thick_gap.shp')
        arcpy.Delete_management('S_WTable_gap.shp')
        arcpy.Delete_management('S_Depth_gap_pcnt.shp')
        arcpy.Delete_management('S_Ksat_gap_pcnt.shp')
        arcpy.Delete_management('S_KsatH_gap_pcnt.shp')
        arcpy.Delete_management('S_KsatL_gap_pcnt.shp')
        arcpy.Delete_management('S_Thick_gap_pcnt.shp')
        arcpy.Delete_management('S_WTable_gap_pcnt.shp')
        arcpy.Delete_management('AllocDep')
        arcpy.Delete_management('AllocKsat')
        arcpy.Delete_management('AllocKsat_H')
        arcpy.Delete_management('AllocKsat_L')
        arcpy.Delete_management('AllocThkGap')
        arcpy.Delete_management('AllocWT')
        arcpy.Delete_management('S_KsatH_est')
        arcpy.Delete_management('S_KsatL_est')
        arcpy.Delete_management('S_Thick_est')
        arcpy.Delete_management('f_int')
        arcpy.Delete_management('f_rast')
        arcpy.Delete_management('f_table')
        arcpy.Delete_management('DEM_slp_int')
        arcpy.Delete_management('DEM_slp_pos')
        #arcpy.Delete_management('S_Trans')
        arcpy.Delete_management('RI_sub_dry')
        arcpy.Delete_management('RI_sub_wet')
        arcpy.Delete_management('Buffer_rise')
        arcpy.Delete_management('Buffer_run')
        arcpy.Delete_management("Buffer_slp")
        arcpy.Delete_management(Manning_n)
        arcpy.Delete_management(Hydraulic_R)
        arcpy.Delete_management('Vel_surf')
        # arcpy.Delete_management('Vel_sub')
        arcpy.Delete_management('Time_surf_i')
        arcpy.Delete_management('Time_surf_t')
        arcpy.Delete_management('Time_surf_r')
        # arcpy.Delete_management('Time_sub_i')
        # arcpy.Delete_management('Time_sub_t')
        # arcpy.Delete_management('Time_sub_r')
    except Exception as err:
        print err
        pass


